/* Caprice32 - Amstrad CPC Emulator
   (c) Copyright 1997-2004 Ulrich Doewich

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define CF  0x01
#define NF	0x02
#define PF	0x04
#define VF	PF
#define XF	0x08
#define HF	0x10
#define YF	0x20
#define ZF	0x40
#define SF	0x80

static word DAATable[0x800] = {
  (0x00<<8)   +ZF         +VF      ,
  (0x01<<8)                        ,
  (0x02<<8)                        ,
  (0x03<<8)               +VF      ,
  (0x04<<8)                        ,
  (0x05<<8)               +VF      ,
  (0x06<<8)               +VF      ,
  (0x07<<8)                        ,
  (0x08<<8)            +XF         ,
  (0x09<<8)            +XF+VF      ,
  (0x10<<8)         +HF            ,
  (0x11<<8)         +HF   +VF      ,
  (0x12<<8)         +HF   +VF      ,
  (0x13<<8)         +HF            ,
  (0x14<<8)         +HF   +VF      ,
  (0x15<<8)         +HF            ,
  (0x10<<8)                        ,
  (0x11<<8)               +VF      ,
  (0x12<<8)               +VF      ,
  (0x13<<8)                        ,
  (0x14<<8)               +VF      ,
  (0x15<<8)                        ,
  (0x16<<8)                        ,
  (0x17<<8)               +VF      ,
  (0x18<<8)            +XF+VF      ,
  (0x19<<8)            +XF         ,
  (0x20<<8)      +YF+HF            ,
  (0x21<<8)      +YF+HF   +VF      ,
  (0x22<<8)      +YF+HF   +VF      ,
  (0x23<<8)      +YF+HF            ,
  (0x24<<8)      +YF+HF   +VF      ,
  (0x25<<8)      +YF+HF            ,
  (0x20<<8)      +YF               ,
  (0x21<<8)      +YF      +VF      ,
  (0x22<<8)      +YF      +VF      ,
  (0x23<<8)      +YF               ,
  (0x24<<8)      +YF      +VF      ,
  (0x25<<8)      +YF               ,
  (0x26<<8)      +YF               ,
  (0x27<<8)      +YF      +VF      ,
  (0x28<<8)      +YF   +XF+VF      ,
  (0x29<<8)      +YF   +XF         ,
  (0x30<<8)      +YF+HF   +VF      ,
  (0x31<<8)      +YF+HF            ,
  (0x32<<8)      +YF+HF            ,
  (0x33<<8)      +YF+HF   +VF      ,
  (0x34<<8)      +YF+HF            ,
  (0x35<<8)      +YF+HF   +VF      ,
  (0x30<<8)      +YF      +VF      ,
  (0x31<<8)      +YF               ,
  (0x32<<8)      +YF               ,
  (0x33<<8)      +YF      +VF      ,
  (0x34<<8)      +YF               ,
  (0x35<<8)      +YF      +VF      ,
  (0x36<<8)      +YF      +VF      ,
  (0x37<<8)      +YF               ,
  (0x38<<8)      +YF   +XF         ,
  (0x39<<8)      +YF   +XF+VF      ,
  (0x40<<8)         +HF            ,
  (0x41<<8)         +HF   +VF      ,
  (0x42<<8)         +HF   +VF      ,
  (0x43<<8)         +HF            ,
  (0x44<<8)         +HF   +VF      ,
  (0x45<<8)         +HF            ,
  (0x40<<8)                        ,
  (0x41<<8)               +VF      ,
  (0x42<<8)               +VF      ,
  (0x43<<8)                        ,
  (0x44<<8)               +VF      ,
  (0x45<<8)                        ,
  (0x46<<8)                        ,
  (0x47<<8)               +VF      ,
  (0x48<<8)            +XF+VF      ,
  (0x49<<8)            +XF         ,
  (0x50<<8)         +HF   +VF      ,
  (0x51<<8)         +HF            ,
  (0x52<<8)         +HF            ,
  (0x53<<8)         +HF   +VF      ,
  (0x54<<8)         +HF            ,
  (0x55<<8)         +HF   +VF      ,
  (0x50<<8)               +VF      ,
  (0x51<<8)                        ,
  (0x52<<8)                        ,
  (0x53<<8)               +VF      ,
  (0x54<<8)                        ,
  (0x55<<8)               +VF      ,
  (0x56<<8)               +VF      ,
  (0x57<<8)                        ,
  (0x58<<8)            +XF         ,
  (0x59<<8)            +XF+VF      ,
  (0x60<<8)      +YF+HF   +VF      ,
  (0x61<<8)      +YF+HF            ,
  (0x62<<8)      +YF+HF            ,
  (0x63<<8)      +YF+HF   +VF      ,
  (0x64<<8)      +YF+HF            ,
  (0x65<<8)      +YF+HF   +VF      ,
  (0x60<<8)      +YF      +VF      ,
  (0x61<<8)      +YF               ,
  (0x62<<8)      +YF               ,
  (0x63<<8)      +YF      +VF      ,
  (0x64<<8)      +YF               ,
  (0x65<<8)      +YF      +VF      ,
  (0x66<<8)      +YF      +VF      ,
  (0x67<<8)      +YF               ,
  (0x68<<8)      +YF   +XF         ,
  (0x69<<8)      +YF   +XF+VF      ,
  (0x70<<8)      +YF+HF            ,
  (0x71<<8)      +YF+HF   +VF      ,
  (0x72<<8)      +YF+HF   +VF      ,
  (0x73<<8)      +YF+HF            ,
  (0x74<<8)      +YF+HF   +VF      ,
  (0x75<<8)      +YF+HF            ,
  (0x70<<8)      +YF               ,
  (0x71<<8)      +YF      +VF      ,
  (0x72<<8)      +YF      +VF      ,
  (0x73<<8)      +YF               ,
  (0x74<<8)      +YF      +VF      ,
  (0x75<<8)      +YF               ,
  (0x76<<8)      +YF               ,
  (0x77<<8)      +YF      +VF      ,
  (0x78<<8)      +YF   +XF+VF      ,
  (0x79<<8)      +YF   +XF         ,
  (0x80<<8)+SF      +HF            ,
  (0x81<<8)+SF      +HF   +VF      ,
  (0x82<<8)+SF      +HF   +VF      ,
  (0x83<<8)+SF      +HF            ,
  (0x84<<8)+SF      +HF   +VF      ,
  (0x85<<8)+SF      +HF            ,
  (0x80<<8)+SF                     ,
  (0x81<<8)+SF            +VF      ,
  (0x82<<8)+SF            +VF      ,
  (0x83<<8)+SF                     ,
  (0x84<<8)+SF            +VF      ,
  (0x85<<8)+SF                     ,
  (0x86<<8)+SF                     ,
  (0x87<<8)+SF            +VF      ,
  (0x88<<8)+SF         +XF+VF      ,
  (0x89<<8)+SF         +XF         ,
  (0x90<<8)+SF      +HF   +VF      ,
  (0x91<<8)+SF      +HF            ,
  (0x92<<8)+SF      +HF            ,
  (0x93<<8)+SF      +HF   +VF      ,
  (0x94<<8)+SF      +HF            ,
  (0x95<<8)+SF      +HF   +VF      ,
  (0x90<<8)+SF            +VF      ,
  (0x91<<8)+SF                     ,
  (0x92<<8)+SF                     ,
  (0x93<<8)+SF            +VF      ,
  (0x94<<8)+SF                     ,
  (0x95<<8)+SF            +VF      ,
  (0x96<<8)+SF            +VF      ,
  (0x97<<8)+SF                     ,
  (0x98<<8)+SF         +XF         ,
  (0x99<<8)+SF         +XF+VF      ,
  (0x00<<8)   +ZF   +HF   +VF   +CF,
  (0x01<<8)         +HF         +CF,
  (0x02<<8)         +HF         +CF,
  (0x03<<8)         +HF   +VF   +CF,
  (0x04<<8)         +HF         +CF,
  (0x05<<8)         +HF   +VF   +CF,
  (0x00<<8)   +ZF         +VF   +CF,
  (0x01<<8)                     +CF,
  (0x02<<8)                     +CF,
  (0x03<<8)               +VF   +CF,
  (0x04<<8)                     +CF,
  (0x05<<8)               +VF   +CF,
  (0x06<<8)               +VF   +CF,
  (0x07<<8)                     +CF,
  (0x08<<8)            +XF      +CF,
  (0x09<<8)            +XF+VF   +CF,
  (0x10<<8)         +HF         +CF,
  (0x11<<8)         +HF   +VF   +CF,
  (0x12<<8)         +HF   +VF   +CF,
  (0x13<<8)         +HF         +CF,
  (0x14<<8)         +HF   +VF   +CF,
  (0x15<<8)         +HF         +CF,
  (0x10<<8)                     +CF,
  (0x11<<8)               +VF   +CF,
  (0x12<<8)               +VF   +CF,
  (0x13<<8)                     +CF,
  (0x14<<8)               +VF   +CF,
  (0x15<<8)                     +CF,
  (0x16<<8)                     +CF,
  (0x17<<8)               +VF   +CF,
  (0x18<<8)            +XF+VF   +CF,
  (0x19<<8)            +XF      +CF,
  (0x20<<8)      +YF+HF         +CF,
  (0x21<<8)      +YF+HF   +VF   +CF,
  (0x22<<8)      +YF+HF   +VF   +CF,
  (0x23<<8)      +YF+HF         +CF,
  (0x24<<8)      +YF+HF   +VF   +CF,
  (0x25<<8)      +YF+HF         +CF,
  (0x20<<8)      +YF            +CF,
  (0x21<<8)      +YF      +VF   +CF,
  (0x22<<8)      +YF      +VF   +CF,
  (0x23<<8)      +YF            +CF,
  (0x24<<8)      +YF      +VF   +CF,
  (0x25<<8)      +YF            +CF,
  (0x26<<8)      +YF            +CF,
  (0x27<<8)      +YF      +VF   +CF,
  (0x28<<8)      +YF   +XF+VF   +CF,
  (0x29<<8)      +YF   +XF      +CF,
  (0x30<<8)      +YF+HF   +VF   +CF,
  (0x31<<8)      +YF+HF         +CF,
  (0x32<<8)      +YF+HF         +CF,
  (0x33<<8)      +YF+HF   +VF   +CF,
  (0x34<<8)      +YF+HF         +CF,
  (0x35<<8)      +YF+HF   +VF   +CF,
  (0x30<<8)      +YF      +VF   +CF,
  (0x31<<8)      +YF            +CF,
  (0x32<<8)      +YF            +CF,
  (0x33<<8)      +YF      +VF   +CF,
  (0x34<<8)      +YF            +CF,
  (0x35<<8)      +YF      +VF   +CF,
  (0x36<<8)      +YF      +VF   +CF,
  (0x37<<8)      +YF            +CF,
  (0x38<<8)      +YF   +XF      +CF,
  (0x39<<8)      +YF   +XF+VF   +CF,
  (0x40<<8)         +HF         +CF,
  (0x41<<8)         +HF   +VF   +CF,
  (0x42<<8)         +HF   +VF   +CF,
  (0x43<<8)         +HF         +CF,
  (0x44<<8)         +HF   +VF   +CF,
  (0x45<<8)         +HF         +CF,
  (0x40<<8)                     +CF,
  (0x41<<8)               +VF   +CF,
  (0x42<<8)               +VF   +CF,
  (0x43<<8)                     +CF,
  (0x44<<8)               +VF   +CF,
  (0x45<<8)                     +CF,
  (0x46<<8)                     +CF,
  (0x47<<8)               +VF   +CF,
  (0x48<<8)            +XF+VF   +CF,
  (0x49<<8)            +XF      +CF,
  (0x50<<8)         +HF   +VF   +CF,
  (0x51<<8)         +HF         +CF,
  (0x52<<8)         +HF         +CF,
  (0x53<<8)         +HF   +VF   +CF,
  (0x54<<8)         +HF         +CF,
  (0x55<<8)         +HF   +VF   +CF,
  (0x50<<8)               +VF   +CF,
  (0x51<<8)                     +CF,
  (0x52<<8)                     +CF,
  (0x53<<8)               +VF   +CF,
  (0x54<<8)                     +CF,
  (0x55<<8)               +VF   +CF,
  (0x56<<8)               +VF   +CF,
  (0x57<<8)                     +CF,
  (0x58<<8)            +XF      +CF,
  (0x59<<8)            +XF+VF   +CF,
  (0x60<<8)      +YF+HF   +VF   +CF,
  (0x61<<8)      +YF+HF         +CF,
  (0x62<<8)      +YF+HF         +CF,
  (0x63<<8)      +YF+HF   +VF   +CF,
  (0x64<<8)      +YF+HF         +CF,
  (0x65<<8)      +YF+HF   +VF   +CF,
  (0x60<<8)      +YF      +VF   +CF,
  (0x61<<8)      +YF            +CF,
  (0x62<<8)      +YF            +CF,
  (0x63<<8)      +YF      +VF   +CF,
  (0x64<<8)      +YF            +CF,
  (0x65<<8)      +YF      +VF   +CF,
  (0x66<<8)      +YF      +VF   +CF,
  (0x67<<8)      +YF            +CF,
  (0x68<<8)      +YF   +XF      +CF,
  (0x69<<8)      +YF   +XF+VF   +CF,
  (0x70<<8)      +YF+HF         +CF,
  (0x71<<8)      +YF+HF   +VF   +CF,
  (0x72<<8)      +YF+HF   +VF   +CF,
  (0x73<<8)      +YF+HF         +CF,
  (0x74<<8)      +YF+HF   +VF   +CF,
  (0x75<<8)      +YF+HF         +CF,
  (0x70<<8)      +YF            +CF,
  (0x71<<8)      +YF      +VF   +CF,
  (0x72<<8)      +YF      +VF   +CF,
  (0x73<<8)      +YF            +CF,
  (0x74<<8)      +YF      +VF   +CF,
  (0x75<<8)      +YF            +CF,
  (0x76<<8)      +YF            +CF,
  (0x77<<8)      +YF      +VF   +CF,
  (0x78<<8)      +YF   +XF+VF   +CF,
  (0x79<<8)      +YF   +XF      +CF,
  (0x80<<8)+SF      +HF         +CF,
  (0x81<<8)+SF      +HF   +VF   +CF,
  (0x82<<8)+SF      +HF   +VF   +CF,
  (0x83<<8)+SF      +HF         +CF,
  (0x84<<8)+SF      +HF   +VF   +CF,
  (0x85<<8)+SF      +HF         +CF,
  (0x80<<8)+SF                  +CF,
  (0x81<<8)+SF            +VF   +CF,
  (0x82<<8)+SF            +VF   +CF,
  (0x83<<8)+SF                  +CF,
  (0x84<<8)+SF            +VF   +CF,
  (0x85<<8)+SF                  +CF,
  (0x86<<8)+SF                  +CF,
  (0x87<<8)+SF            +VF   +CF,
  (0x88<<8)+SF         +XF+VF   +CF,
  (0x89<<8)+SF         +XF      +CF,
  (0x90<<8)+SF      +HF   +VF   +CF,
  (0x91<<8)+SF      +HF         +CF,
  (0x92<<8)+SF      +HF         +CF,
  (0x93<<8)+SF      +HF   +VF   +CF,
  (0x94<<8)+SF      +HF         +CF,
  (0x95<<8)+SF      +HF   +VF   +CF,
  (0x90<<8)+SF            +VF   +CF,
  (0x91<<8)+SF                  +CF,
  (0x92<<8)+SF                  +CF,
  (0x93<<8)+SF            +VF   +CF,
  (0x94<<8)+SF                  +CF,
  (0x95<<8)+SF            +VF   +CF,
  (0x96<<8)+SF            +VF   +CF,
  (0x97<<8)+SF                  +CF,
  (0x98<<8)+SF         +XF      +CF,
  (0x99<<8)+SF         +XF+VF   +CF,
  (0xA0<<8)+SF   +YF+HF   +VF   +CF,
  (0xA1<<8)+SF   +YF+HF         +CF,
  (0xA2<<8)+SF   +YF+HF         +CF,
  (0xA3<<8)+SF   +YF+HF   +VF   +CF,
  (0xA4<<8)+SF   +YF+HF         +CF,
  (0xA5<<8)+SF   +YF+HF   +VF   +CF,
  (0xA0<<8)+SF   +YF      +VF   +CF,
  (0xA1<<8)+SF   +YF            +CF,
  (0xA2<<8)+SF   +YF            +CF,
  (0xA3<<8)+SF   +YF      +VF   +CF,
  (0xA4<<8)+SF   +YF            +CF,
  (0xA5<<8)+SF   +YF      +VF   +CF,
  (0xA6<<8)+SF   +YF      +VF   +CF,
  (0xA7<<8)+SF   +YF            +CF,
  (0xA8<<8)+SF   +YF   +XF      +CF,
  (0xA9<<8)+SF   +YF   +XF+VF   +CF,
  (0xB0<<8)+SF   +YF+HF         +CF,
  (0xB1<<8)+SF   +YF+HF   +VF   +CF,
  (0xB2<<8)+SF   +YF+HF   +VF   +CF,
  (0xB3<<8)+SF   +YF+HF         +CF,
  (0xB4<<8)+SF   +YF+HF   +VF   +CF,
  (0xB5<<8)+SF   +YF+HF         +CF,
  (0xB0<<8)+SF   +YF            +CF,
  (0xB1<<8)+SF   +YF      +VF   +CF,
  (0xB2<<8)+SF   +YF      +VF   +CF,
  (0xB3<<8)+SF   +YF            +CF,
  (0xB4<<8)+SF   +YF      +VF   +CF,
  (0xB5<<8)+SF   +YF            +CF,
  (0xB6<<8)+SF   +YF            +CF,
  (0xB7<<8)+SF   +YF      +VF   +CF,
  (0xB8<<8)+SF   +YF   +XF+VF   +CF,
  (0xB9<<8)+SF   +YF   +XF      +CF,
  (0xC0<<8)+SF      +HF   +VF   +CF,
  (0xC1<<8)+SF      +HF         +CF,
  (0xC2<<8)+SF      +HF         +CF,
  (0xC3<<8)+SF      +HF   +VF   +CF,
  (0xC4<<8)+SF      +HF         +CF,
  (0xC5<<8)+SF      +HF   +VF   +CF,
  (0xC0<<8)+SF            +VF   +CF,
  (0xC1<<8)+SF                  +CF,
  (0xC2<<8)+SF                  +CF,
  (0xC3<<8)+SF            +VF   +CF,
  (0xC4<<8)+SF                  +CF,
  (0xC5<<8)+SF            +VF   +CF,
  (0xC6<<8)+SF            +VF   +CF,
  (0xC7<<8)+SF                  +CF,
  (0xC8<<8)+SF         +XF      +CF,
  (0xC9<<8)+SF         +XF+VF   +CF,
  (0xD0<<8)+SF      +HF         +CF,
  (0xD1<<8)+SF      +HF   +VF   +CF,
  (0xD2<<8)+SF      +HF   +VF   +CF,
  (0xD3<<8)+SF      +HF         +CF,
  (0xD4<<8)+SF      +HF   +VF   +CF,
  (0xD5<<8)+SF      +HF         +CF,
  (0xD0<<8)+SF                  +CF,
  (0xD1<<8)+SF            +VF   +CF,
  (0xD2<<8)+SF            +VF   +CF,
  (0xD3<<8)+SF                  +CF,
  (0xD4<<8)+SF            +VF   +CF,
  (0xD5<<8)+SF                  +CF,
  (0xD6<<8)+SF                  +CF,
  (0xD7<<8)+SF            +VF   +CF,
  (0xD8<<8)+SF         +XF+VF   +CF,
  (0xD9<<8)+SF         +XF      +CF,
  (0xE0<<8)+SF   +YF+HF         +CF,
  (0xE1<<8)+SF   +YF+HF   +VF   +CF,
  (0xE2<<8)+SF   +YF+HF   +VF   +CF,
  (0xE3<<8)+SF   +YF+HF         +CF,
  (0xE4<<8)+SF   +YF+HF   +VF   +CF,
  (0xE5<<8)+SF   +YF+HF         +CF,
  (0xE0<<8)+SF   +YF            +CF,
  (0xE1<<8)+SF   +YF      +VF   +CF,
  (0xE2<<8)+SF   +YF      +VF   +CF,
  (0xE3<<8)+SF   +YF            +CF,
  (0xE4<<8)+SF   +YF      +VF   +CF,
  (0xE5<<8)+SF   +YF            +CF,
  (0xE6<<8)+SF   +YF            +CF,
  (0xE7<<8)+SF   +YF      +VF   +CF,
  (0xE8<<8)+SF   +YF   +XF+VF   +CF,
  (0xE9<<8)+SF   +YF   +XF      +CF,
  (0xF0<<8)+SF   +YF+HF   +VF   +CF,
  (0xF1<<8)+SF   +YF+HF         +CF,
  (0xF2<<8)+SF   +YF+HF         +CF,
  (0xF3<<8)+SF   +YF+HF   +VF   +CF,
  (0xF4<<8)+SF   +YF+HF         +CF,
  (0xF5<<8)+SF   +YF+HF   +VF   +CF,
  (0xF0<<8)+SF   +YF      +VF   +CF,
  (0xF1<<8)+SF   +YF            +CF,
  (0xF2<<8)+SF   +YF            +CF,
  (0xF3<<8)+SF   +YF      +VF   +CF,
  (0xF4<<8)+SF   +YF            +CF,
  (0xF5<<8)+SF   +YF      +VF   +CF,
  (0xF6<<8)+SF   +YF      +VF   +CF,
  (0xF7<<8)+SF   +YF            +CF,
  (0xF8<<8)+SF   +YF   +XF      +CF,
  (0xF9<<8)+SF   +YF   +XF+VF   +CF,
  (0x00<<8)   +ZF   +HF   +VF   +CF,
  (0x01<<8)         +HF         +CF,
  (0x02<<8)         +HF         +CF,
  (0x03<<8)         +HF   +VF   +CF,
  (0x04<<8)         +HF         +CF,
  (0x05<<8)         +HF   +VF   +CF,
  (0x00<<8)   +ZF         +VF   +CF,
  (0x01<<8)                     +CF,
  (0x02<<8)                     +CF,
  (0x03<<8)               +VF   +CF,
  (0x04<<8)                     +CF,
  (0x05<<8)               +VF   +CF,
  (0x06<<8)               +VF   +CF,
  (0x07<<8)                     +CF,
  (0x08<<8)            +XF      +CF,
  (0x09<<8)            +XF+VF   +CF,
  (0x10<<8)         +HF         +CF,
  (0x11<<8)         +HF   +VF   +CF,
  (0x12<<8)         +HF   +VF   +CF,
  (0x13<<8)         +HF         +CF,
  (0x14<<8)         +HF   +VF   +CF,
  (0x15<<8)         +HF         +CF,
  (0x10<<8)                     +CF,
  (0x11<<8)               +VF   +CF,
  (0x12<<8)               +VF   +CF,
  (0x13<<8)                     +CF,
  (0x14<<8)               +VF   +CF,
  (0x15<<8)                     +CF,
  (0x16<<8)                     +CF,
  (0x17<<8)               +VF   +CF,
  (0x18<<8)            +XF+VF   +CF,
  (0x19<<8)            +XF      +CF,
  (0x20<<8)      +YF+HF         +CF,
  (0x21<<8)      +YF+HF   +VF   +CF,
  (0x22<<8)      +YF+HF   +VF   +CF,
  (0x23<<8)      +YF+HF         +CF,
  (0x24<<8)      +YF+HF   +VF   +CF,
  (0x25<<8)      +YF+HF         +CF,
  (0x20<<8)      +YF            +CF,
  (0x21<<8)      +YF      +VF   +CF,
  (0x22<<8)      +YF      +VF   +CF,
  (0x23<<8)      +YF            +CF,
  (0x24<<8)      +YF      +VF   +CF,
  (0x25<<8)      +YF            +CF,
  (0x26<<8)      +YF            +CF,
  (0x27<<8)      +YF      +VF   +CF,
  (0x28<<8)      +YF   +XF+VF   +CF,
  (0x29<<8)      +YF   +XF      +CF,
  (0x30<<8)      +YF+HF   +VF   +CF,
  (0x31<<8)      +YF+HF         +CF,
  (0x32<<8)      +YF+HF         +CF,
  (0x33<<8)      +YF+HF   +VF   +CF,
  (0x34<<8)      +YF+HF         +CF,
  (0x35<<8)      +YF+HF   +VF   +CF,
  (0x30<<8)      +YF      +VF   +CF,
  (0x31<<8)      +YF            +CF,
  (0x32<<8)      +YF            +CF,
  (0x33<<8)      +YF      +VF   +CF,
  (0x34<<8)      +YF            +CF,
  (0x35<<8)      +YF      +VF   +CF,
  (0x36<<8)      +YF      +VF   +CF,
  (0x37<<8)      +YF            +CF,
  (0x38<<8)      +YF   +XF      +CF,
  (0x39<<8)      +YF   +XF+VF   +CF,
  (0x40<<8)         +HF         +CF,
  (0x41<<8)         +HF   +VF   +CF,
  (0x42<<8)         +HF   +VF   +CF,
  (0x43<<8)         +HF         +CF,
  (0x44<<8)         +HF   +VF   +CF,
  (0x45<<8)         +HF         +CF,
  (0x40<<8)                     +CF,
  (0x41<<8)               +VF   +CF,
  (0x42<<8)               +VF   +CF,
  (0x43<<8)                     +CF,
  (0x44<<8)               +VF   +CF,
  (0x45<<8)                     +CF,
  (0x46<<8)                     +CF,
  (0x47<<8)               +VF   +CF,
  (0x48<<8)            +XF+VF   +CF,
  (0x49<<8)            +XF      +CF,
  (0x50<<8)         +HF   +VF   +CF,
  (0x51<<8)         +HF         +CF,
  (0x52<<8)         +HF         +CF,
  (0x53<<8)         +HF   +VF   +CF,
  (0x54<<8)         +HF         +CF,
  (0x55<<8)         +HF   +VF   +CF,
  (0x50<<8)               +VF   +CF,
  (0x51<<8)                     +CF,
  (0x52<<8)                     +CF,
  (0x53<<8)               +VF   +CF,
  (0x54<<8)                     +CF,
  (0x55<<8)               +VF   +CF,
  (0x56<<8)               +VF   +CF,
  (0x57<<8)                     +CF,
  (0x58<<8)            +XF      +CF,
  (0x59<<8)            +XF+VF   +CF,
  (0x60<<8)      +YF+HF   +VF   +CF,
  (0x61<<8)      +YF+HF         +CF,
  (0x62<<8)      +YF+HF         +CF,
  (0x63<<8)      +YF+HF   +VF   +CF,
  (0x64<<8)      +YF+HF         +CF,
  (0x65<<8)      +YF+HF   +VF   +CF,
  (0x06<<8)               +VF      ,
  (0x07<<8)                        ,
  (0x08<<8)            +XF         ,
  (0x09<<8)            +XF+VF      ,
  (0x0A<<8)            +XF+VF      ,
  (0x0B<<8)            +XF         ,
  (0x0C<<8)            +XF+VF      ,
  (0x0D<<8)            +XF         ,
  (0x0E<<8)            +XF         ,
  (0x0F<<8)            +XF+VF      ,
  (0x10<<8)         +HF            ,
  (0x11<<8)         +HF   +VF      ,
  (0x12<<8)         +HF   +VF      ,
  (0x13<<8)         +HF            ,
  (0x14<<8)         +HF   +VF      ,
  (0x15<<8)         +HF            ,
  (0x16<<8)                        ,
  (0x17<<8)               +VF      ,
  (0x18<<8)            +XF+VF      ,
  (0x19<<8)            +XF         ,
  (0x1A<<8)            +XF         ,
  (0x1B<<8)            +XF+VF      ,
  (0x1C<<8)            +XF         ,
  (0x1D<<8)            +XF+VF      ,
  (0x1E<<8)            +XF+VF      ,
  (0x1F<<8)            +XF         ,
  (0x20<<8)      +YF+HF            ,
  (0x21<<8)      +YF+HF   +VF      ,
  (0x22<<8)      +YF+HF   +VF      ,
  (0x23<<8)      +YF+HF            ,
  (0x24<<8)      +YF+HF   +VF      ,
  (0x25<<8)      +YF+HF            ,
  (0x26<<8)      +YF               ,
  (0x27<<8)      +YF      +VF      ,
  (0x28<<8)      +YF   +XF+VF      ,
  (0x29<<8)      +YF   +XF         ,
  (0x2A<<8)      +YF   +XF         ,
  (0x2B<<8)      +YF   +XF+VF      ,
  (0x2C<<8)      +YF   +XF         ,
  (0x2D<<8)      +YF   +XF+VF      ,
  (0x2E<<8)      +YF   +XF+VF      ,
  (0x2F<<8)      +YF   +XF         ,
  (0x30<<8)      +YF+HF   +VF      ,
  (0x31<<8)      +YF+HF            ,
  (0x32<<8)      +YF+HF            ,
  (0x33<<8)      +YF+HF   +VF      ,
  (0x34<<8)      +YF+HF            ,
  (0x35<<8)      +YF+HF   +VF      ,
  (0x36<<8)      +YF      +VF      ,
  (0x37<<8)      +YF               ,
  (0x38<<8)      +YF   +XF         ,
  (0x39<<8)      +YF   +XF+VF      ,
  (0x3A<<8)      +YF   +XF+VF      ,
  (0x3B<<8)      +YF   +XF         ,
  (0x3C<<8)      +YF   +XF+VF      ,
  (0x3D<<8)      +YF   +XF         ,
  (0x3E<<8)      +YF   +XF         ,
  (0x3F<<8)      +YF   +XF+VF      ,
  (0x40<<8)         +HF            ,
  (0x41<<8)         +HF   +VF      ,
  (0x42<<8)         +HF   +VF      ,
  (0x43<<8)         +HF            ,
  (0x44<<8)         +HF   +VF      ,
  (0x45<<8)         +HF            ,
  (0x46<<8)                        ,
  (0x47<<8)               +VF      ,
  (0x48<<8)            +XF+VF      ,
  (0x49<<8)            +XF         ,
  (0x4A<<8)            +XF         ,
  (0x4B<<8)            +XF+VF      ,
  (0x4C<<8)            +XF         ,
  (0x4D<<8)            +XF+VF      ,
  (0x4E<<8)            +XF+VF      ,
  (0x4F<<8)            +XF         ,
  (0x50<<8)         +HF   +VF      ,
  (0x51<<8)         +HF            ,
  (0x52<<8)         +HF            ,
  (0x53<<8)         +HF   +VF      ,
  (0x54<<8)         +HF            ,
  (0x55<<8)         +HF   +VF      ,
  (0x56<<8)               +VF      ,
  (0x57<<8)                        ,
  (0x58<<8)            +XF         ,
  (0x59<<8)            +XF+VF      ,
  (0x5A<<8)            +XF+VF      ,
  (0x5B<<8)            +XF         ,
  (0x5C<<8)            +XF+VF      ,
  (0x5D<<8)            +XF         ,
  (0x5E<<8)            +XF         ,
  (0x5F<<8)            +XF+VF      ,
  (0x60<<8)      +YF+HF   +VF      ,
  (0x61<<8)      +YF+HF            ,
  (0x62<<8)      +YF+HF            ,
  (0x63<<8)      +YF+HF   +VF      ,
  (0x64<<8)      +YF+HF            ,
  (0x65<<8)      +YF+HF   +VF      ,
  (0x66<<8)      +YF      +VF      ,
  (0x67<<8)      +YF               ,
  (0x68<<8)      +YF   +XF         ,
  (0x69<<8)      +YF   +XF+VF      ,
  (0x6A<<8)      +YF   +XF+VF      ,
  (0x6B<<8)      +YF   +XF         ,
  (0x6C<<8)      +YF   +XF+VF      ,
  (0x6D<<8)      +YF   +XF         ,
  (0x6E<<8)      +YF   +XF         ,
  (0x6F<<8)      +YF   +XF+VF      ,
  (0x70<<8)      +YF+HF            ,
  (0x71<<8)      +YF+HF   +VF      ,
  (0x72<<8)      +YF+HF   +VF      ,
  (0x73<<8)      +YF+HF            ,
  (0x74<<8)      +YF+HF   +VF      ,
  (0x75<<8)      +YF+HF            ,
  (0x76<<8)      +YF               ,
  (0x77<<8)      +YF      +VF      ,
  (0x78<<8)      +YF   +XF+VF      ,
  (0x79<<8)      +YF   +XF         ,
  (0x7A<<8)      +YF   +XF         ,
  (0x7B<<8)      +YF   +XF+VF      ,
  (0x7C<<8)      +YF   +XF         ,
  (0x7D<<8)      +YF   +XF+VF      ,
  (0x7E<<8)      +YF   +XF+VF      ,
  (0x7F<<8)      +YF   +XF         ,
  (0x80<<8)+SF      +HF            ,
  (0x81<<8)+SF      +HF   +VF      ,
  (0x82<<8)+SF      +HF   +VF      ,
  (0x83<<8)+SF      +HF            ,
  (0x84<<8)+SF      +HF   +VF      ,
  (0x85<<8)+SF      +HF            ,
  (0x86<<8)+SF                     ,
  (0x87<<8)+SF            +VF      ,
  (0x88<<8)+SF         +XF+VF      ,
  (0x89<<8)+SF         +XF         ,
  (0x8A<<8)+SF         +XF         ,
  (0x8B<<8)+SF         +XF+VF      ,
  (0x8C<<8)+SF         +XF         ,
  (0x8D<<8)+SF         +XF+VF      ,
  (0x8E<<8)+SF         +XF+VF      ,
  (0x8F<<8)+SF         +XF         ,
  (0x90<<8)+SF      +HF   +VF      ,
  (0x91<<8)+SF      +HF            ,
  (0x92<<8)+SF      +HF            ,
  (0x93<<8)+SF      +HF   +VF      ,
  (0x94<<8)+SF      +HF            ,
  (0x95<<8)+SF      +HF   +VF      ,
  (0x96<<8)+SF            +VF      ,
  (0x97<<8)+SF                     ,
  (0x98<<8)+SF         +XF         ,
  (0x99<<8)+SF         +XF+VF      ,
  (0x9A<<8)+SF         +XF+VF      ,
  (0x9B<<8)+SF         +XF         ,
  (0x9C<<8)+SF         +XF+VF      ,
  (0x9D<<8)+SF         +XF         ,
  (0x9E<<8)+SF         +XF         ,
  (0x9F<<8)+SF         +XF+VF      ,
  (0x00<<8)   +ZF   +HF   +VF   +CF,
  (0x01<<8)         +HF         +CF,
  (0x02<<8)         +HF         +CF,
  (0x03<<8)         +HF   +VF   +CF,
  (0x04<<8)         +HF         +CF,
  (0x05<<8)         +HF   +VF   +CF,
  (0x06<<8)               +VF   +CF,
  (0x07<<8)                     +CF,
  (0x08<<8)            +XF      +CF,
  (0x09<<8)            +XF+VF   +CF,
  (0x0A<<8)            +XF+VF   +CF,
  (0x0B<<8)            +XF      +CF,
  (0x0C<<8)            +XF+VF   +CF,
  (0x0D<<8)            +XF      +CF,
  (0x0E<<8)            +XF      +CF,
  (0x0F<<8)            +XF+VF   +CF,
  (0x10<<8)         +HF         +CF,
  (0x11<<8)         +HF   +VF   +CF,
  (0x12<<8)         +HF   +VF   +CF,
  (0x13<<8)         +HF         +CF,
  (0x14<<8)         +HF   +VF   +CF,
  (0x15<<8)         +HF         +CF,
  (0x16<<8)                     +CF,
  (0x17<<8)               +VF   +CF,
  (0x18<<8)            +XF+VF   +CF,
  (0x19<<8)            +XF      +CF,
  (0x1A<<8)            +XF      +CF,
  (0x1B<<8)            +XF+VF   +CF,
  (0x1C<<8)            +XF      +CF,
  (0x1D<<8)            +XF+VF   +CF,
  (0x1E<<8)            +XF+VF   +CF,
  (0x1F<<8)            +XF      +CF,
  (0x20<<8)      +YF+HF         +CF,
  (0x21<<8)      +YF+HF   +VF   +CF,
  (0x22<<8)      +YF+HF   +VF   +CF,
  (0x23<<8)      +YF+HF         +CF,
  (0x24<<8)      +YF+HF   +VF   +CF,
  (0x25<<8)      +YF+HF         +CF,
  (0x26<<8)      +YF            +CF,
  (0x27<<8)      +YF      +VF   +CF,
  (0x28<<8)      +YF   +XF+VF   +CF,
  (0x29<<8)      +YF   +XF      +CF,
  (0x2A<<8)      +YF   +XF      +CF,
  (0x2B<<8)      +YF   +XF+VF   +CF,
  (0x2C<<8)      +YF   +XF      +CF,
  (0x2D<<8)      +YF   +XF+VF   +CF,
  (0x2E<<8)      +YF   +XF+VF   +CF,
  (0x2F<<8)      +YF   +XF      +CF,
  (0x30<<8)      +YF+HF   +VF   +CF,
  (0x31<<8)      +YF+HF         +CF,
  (0x32<<8)      +YF+HF         +CF,
  (0x33<<8)      +YF+HF   +VF   +CF,
  (0x34<<8)      +YF+HF         +CF,
  (0x35<<8)      +YF+HF   +VF   +CF,
  (0x36<<8)      +YF      +VF   +CF,
  (0x37<<8)      +YF            +CF,
  (0x38<<8)      +YF   +XF      +CF,
  (0x39<<8)      +YF   +XF+VF   +CF,
  (0x3A<<8)      +YF   +XF+VF   +CF,
  (0x3B<<8)      +YF   +XF      +CF,
  (0x3C<<8)      +YF   +XF+VF   +CF,
  (0x3D<<8)      +YF   +XF      +CF,
  (0x3E<<8)      +YF   +XF      +CF,
  (0x3F<<8)      +YF   +XF+VF   +CF,
  (0x40<<8)         +HF         +CF,
  (0x41<<8)         +HF   +VF   +CF,
  (0x42<<8)         +HF   +VF   +CF,
  (0x43<<8)         +HF         +CF,
  (0x44<<8)         +HF   +VF   +CF,
  (0x45<<8)         +HF         +CF,
  (0x46<<8)                     +CF,
  (0x47<<8)               +VF   +CF,
  (0x48<<8)            +XF+VF   +CF,
  (0x49<<8)            +XF      +CF,
  (0x4A<<8)            +XF      +CF,
  (0x4B<<8)            +XF+VF   +CF,
  (0x4C<<8)            +XF      +CF,
  (0x4D<<8)            +XF+VF   +CF,
  (0x4E<<8)            +XF+VF   +CF,
  (0x4F<<8)            +XF      +CF,
  (0x50<<8)         +HF   +VF   +CF,
  (0x51<<8)         +HF         +CF,
  (0x52<<8)         +HF         +CF,
  (0x53<<8)         +HF   +VF   +CF,
  (0x54<<8)         +HF         +CF,
  (0x55<<8)         +HF   +VF   +CF,
  (0x56<<8)               +VF   +CF,
  (0x57<<8)                     +CF,
  (0x58<<8)            +XF      +CF,
  (0x59<<8)            +XF+VF   +CF,
  (0x5A<<8)            +XF+VF   +CF,
  (0x5B<<8)            +XF      +CF,
  (0x5C<<8)            +XF+VF   +CF,
  (0x5D<<8)            +XF      +CF,
  (0x5E<<8)            +XF      +CF,
  (0x5F<<8)            +XF+VF   +CF,
  (0x60<<8)      +YF+HF   +VF   +CF,
  (0x61<<8)      +YF+HF         +CF,
  (0x62<<8)      +YF+HF         +CF,
  (0x63<<8)      +YF+HF   +VF   +CF,
  (0x64<<8)      +YF+HF         +CF,
  (0x65<<8)      +YF+HF   +VF   +CF,
  (0x66<<8)      +YF      +VF   +CF,
  (0x67<<8)      +YF            +CF,
  (0x68<<8)      +YF   +XF      +CF,
  (0x69<<8)      +YF   +XF+VF   +CF,
  (0x6A<<8)      +YF   +XF+VF   +CF,
  (0x6B<<8)      +YF   +XF      +CF,
  (0x6C<<8)      +YF   +XF+VF   +CF,
  (0x6D<<8)      +YF   +XF      +CF,
  (0x6E<<8)      +YF   +XF      +CF,
  (0x6F<<8)      +YF   +XF+VF   +CF,
  (0x70<<8)      +YF+HF         +CF,
  (0x71<<8)      +YF+HF   +VF   +CF,
  (0x72<<8)      +YF+HF   +VF   +CF,
  (0x73<<8)      +YF+HF         +CF,
  (0x74<<8)      +YF+HF   +VF   +CF,
  (0x75<<8)      +YF+HF         +CF,
  (0x76<<8)      +YF            +CF,
  (0x77<<8)      +YF      +VF   +CF,
  (0x78<<8)      +YF   +XF+VF   +CF,
  (0x79<<8)      +YF   +XF      +CF,
  (0x7A<<8)      +YF   +XF      +CF,
  (0x7B<<8)      +YF   +XF+VF   +CF,
  (0x7C<<8)      +YF   +XF      +CF,
  (0x7D<<8)      +YF   +XF+VF   +CF,
  (0x7E<<8)      +YF   +XF+VF   +CF,
  (0x7F<<8)      +YF   +XF      +CF,
  (0x80<<8)+SF      +HF         +CF,
  (0x81<<8)+SF      +HF   +VF   +CF,
  (0x82<<8)+SF      +HF   +VF   +CF,
  (0x83<<8)+SF      +HF         +CF,
  (0x84<<8)+SF      +HF   +VF   +CF,
  (0x85<<8)+SF      +HF         +CF,
  (0x86<<8)+SF                  +CF,
  (0x87<<8)+SF            +VF   +CF,
  (0x88<<8)+SF         +XF+VF   +CF,
  (0x89<<8)+SF         +XF      +CF,
  (0x8A<<8)+SF         +XF      +CF,
  (0x8B<<8)+SF         +XF+VF   +CF,
  (0x8C<<8)+SF         +XF      +CF,
  (0x8D<<8)+SF         +XF+VF   +CF,
  (0x8E<<8)+SF         +XF+VF   +CF,
  (0x8F<<8)+SF         +XF      +CF,
  (0x90<<8)+SF      +HF   +VF   +CF,
  (0x91<<8)+SF      +HF         +CF,
  (0x92<<8)+SF      +HF         +CF,
  (0x93<<8)+SF      +HF   +VF   +CF,
  (0x94<<8)+SF      +HF         +CF,
  (0x95<<8)+SF      +HF   +VF   +CF,
  (0x96<<8)+SF            +VF   +CF,
  (0x97<<8)+SF                  +CF,
  (0x98<<8)+SF         +XF      +CF,
  (0x99<<8)+SF         +XF+VF   +CF,
  (0x9A<<8)+SF         +XF+VF   +CF,
  (0x9B<<8)+SF         +XF      +CF,
  (0x9C<<8)+SF         +XF+VF   +CF,
  (0x9D<<8)+SF         +XF      +CF,
  (0x9E<<8)+SF         +XF      +CF,
  (0x9F<<8)+SF         +XF+VF   +CF,
  (0xA0<<8)+SF   +YF+HF   +VF   +CF,
  (0xA1<<8)+SF   +YF+HF         +CF,
  (0xA2<<8)+SF   +YF+HF         +CF,
  (0xA3<<8)+SF   +YF+HF   +VF   +CF,
  (0xA4<<8)+SF   +YF+HF         +CF,
  (0xA5<<8)+SF   +YF+HF   +VF   +CF,
  (0xA6<<8)+SF   +YF      +VF   +CF,
  (0xA7<<8)+SF   +YF            +CF,
  (0xA8<<8)+SF   +YF   +XF      +CF,
  (0xA9<<8)+SF   +YF   +XF+VF   +CF,
  (0xAA<<8)+SF   +YF   +XF+VF   +CF,
  (0xAB<<8)+SF   +YF   +XF      +CF,
  (0xAC<<8)+SF   +YF   +XF+VF   +CF,
  (0xAD<<8)+SF   +YF   +XF      +CF,
  (0xAE<<8)+SF   +YF   +XF      +CF,
  (0xAF<<8)+SF   +YF   +XF+VF   +CF,
  (0xB0<<8)+SF   +YF+HF         +CF,
  (0xB1<<8)+SF   +YF+HF   +VF   +CF,
  (0xB2<<8)+SF   +YF+HF   +VF   +CF,
  (0xB3<<8)+SF   +YF+HF         +CF,
  (0xB4<<8)+SF   +YF+HF   +VF   +CF,
  (0xB5<<8)+SF   +YF+HF         +CF,
  (0xB6<<8)+SF   +YF            +CF,
  (0xB7<<8)+SF   +YF      +VF   +CF,
  (0xB8<<8)+SF   +YF   +XF+VF   +CF,
  (0xB9<<8)+SF   +YF   +XF      +CF,
  (0xBA<<8)+SF   +YF   +XF      +CF,
  (0xBB<<8)+SF   +YF   +XF+VF   +CF,
  (0xBC<<8)+SF   +YF   +XF      +CF,
  (0xBD<<8)+SF   +YF   +XF+VF   +CF,
  (0xBE<<8)+SF   +YF   +XF+VF   +CF,
  (0xBF<<8)+SF   +YF   +XF      +CF,
  (0xC0<<8)+SF      +HF   +VF   +CF,
  (0xC1<<8)+SF      +HF         +CF,
  (0xC2<<8)+SF      +HF         +CF,
  (0xC3<<8)+SF      +HF   +VF   +CF,
  (0xC4<<8)+SF      +HF         +CF,
  (0xC5<<8)+SF      +HF   +VF   +CF,
  (0xC6<<8)+SF            +VF   +CF,
  (0xC7<<8)+SF                  +CF,
  (0xC8<<8)+SF         +XF      +CF,
  (0xC9<<8)+SF         +XF+VF   +CF,
  (0xCA<<8)+SF         +XF+VF   +CF,
  (0xCB<<8)+SF         +XF      +CF,
  (0xCC<<8)+SF         +XF+VF   +CF,
  (0xCD<<8)+SF         +XF      +CF,
  (0xCE<<8)+SF         +XF      +CF,
  (0xCF<<8)+SF         +XF+VF   +CF,
  (0xD0<<8)+SF      +HF         +CF,
  (0xD1<<8)+SF      +HF   +VF   +CF,
  (0xD2<<8)+SF      +HF   +VF   +CF,
  (0xD3<<8)+SF      +HF         +CF,
  (0xD4<<8)+SF      +HF   +VF   +CF,
  (0xD5<<8)+SF      +HF         +CF,
  (0xD6<<8)+SF                  +CF,
  (0xD7<<8)+SF            +VF   +CF,
  (0xD8<<8)+SF         +XF+VF   +CF,
  (0xD9<<8)+SF         +XF      +CF,
  (0xDA<<8)+SF         +XF      +CF,
  (0xDB<<8)+SF         +XF+VF   +CF,
  (0xDC<<8)+SF         +XF      +CF,
  (0xDD<<8)+SF         +XF+VF   +CF,
  (0xDE<<8)+SF         +XF+VF   +CF,
  (0xDF<<8)+SF         +XF      +CF,
  (0xE0<<8)+SF   +YF+HF         +CF,
  (0xE1<<8)+SF   +YF+HF   +VF   +CF,
  (0xE2<<8)+SF   +YF+HF   +VF   +CF,
  (0xE3<<8)+SF   +YF+HF         +CF,
  (0xE4<<8)+SF   +YF+HF   +VF   +CF,
  (0xE5<<8)+SF   +YF+HF         +CF,
  (0xE6<<8)+SF   +YF            +CF,
  (0xE7<<8)+SF   +YF      +VF   +CF,
  (0xE8<<8)+SF   +YF   +XF+VF   +CF,
  (0xE9<<8)+SF   +YF   +XF      +CF,
  (0xEA<<8)+SF   +YF   +XF      +CF,
  (0xEB<<8)+SF   +YF   +XF+VF   +CF,
  (0xEC<<8)+SF   +YF   +XF      +CF,
  (0xED<<8)+SF   +YF   +XF+VF   +CF,
  (0xEE<<8)+SF   +YF   +XF+VF   +CF,
  (0xEF<<8)+SF   +YF   +XF      +CF,
  (0xF0<<8)+SF   +YF+HF   +VF   +CF,
  (0xF1<<8)+SF   +YF+HF         +CF,
  (0xF2<<8)+SF   +YF+HF         +CF,
  (0xF3<<8)+SF   +YF+HF   +VF   +CF,
  (0xF4<<8)+SF   +YF+HF         +CF,
  (0xF5<<8)+SF   +YF+HF   +VF   +CF,
  (0xF6<<8)+SF   +YF      +VF   +CF,
  (0xF7<<8)+SF   +YF            +CF,
  (0xF8<<8)+SF   +YF   +XF      +CF,
  (0xF9<<8)+SF   +YF   +XF+VF   +CF,
  (0xFA<<8)+SF   +YF   +XF+VF   +CF,
  (0xFB<<8)+SF   +YF   +XF      +CF,
  (0xFC<<8)+SF   +YF   +XF+VF   +CF,
  (0xFD<<8)+SF   +YF   +XF      +CF,
  (0xFE<<8)+SF   +YF   +XF      +CF,
  (0xFF<<8)+SF   +YF   +XF+VF   +CF,
  (0x00<<8)   +ZF   +HF   +VF   +CF,
  (0x01<<8)         +HF         +CF,
  (0x02<<8)         +HF         +CF,
  (0x03<<8)         +HF   +VF   +CF,
  (0x04<<8)         +HF         +CF,
  (0x05<<8)         +HF   +VF   +CF,
  (0x06<<8)               +VF   +CF,
  (0x07<<8)                     +CF,
  (0x08<<8)            +XF      +CF,
  (0x09<<8)            +XF+VF   +CF,
  (0x0A<<8)            +XF+VF   +CF,
  (0x0B<<8)            +XF      +CF,
  (0x0C<<8)            +XF+VF   +CF,
  (0x0D<<8)            +XF      +CF,
  (0x0E<<8)            +XF      +CF,
  (0x0F<<8)            +XF+VF   +CF,
  (0x10<<8)         +HF         +CF,
  (0x11<<8)         +HF   +VF   +CF,
  (0x12<<8)         +HF   +VF   +CF,
  (0x13<<8)         +HF         +CF,
  (0x14<<8)         +HF   +VF   +CF,
  (0x15<<8)         +HF         +CF,
  (0x16<<8)                     +CF,
  (0x17<<8)               +VF   +CF,
  (0x18<<8)            +XF+VF   +CF,
  (0x19<<8)            +XF      +CF,
  (0x1A<<8)            +XF      +CF,
  (0x1B<<8)            +XF+VF   +CF,
  (0x1C<<8)            +XF      +CF,
  (0x1D<<8)            +XF+VF   +CF,
  (0x1E<<8)            +XF+VF   +CF,
  (0x1F<<8)            +XF      +CF,
  (0x20<<8)      +YF+HF         +CF,
  (0x21<<8)      +YF+HF   +VF   +CF,
  (0x22<<8)      +YF+HF   +VF   +CF,
  (0x23<<8)      +YF+HF         +CF,
  (0x24<<8)      +YF+HF   +VF   +CF,
  (0x25<<8)      +YF+HF         +CF,
  (0x26<<8)      +YF            +CF,
  (0x27<<8)      +YF      +VF   +CF,
  (0x28<<8)      +YF   +XF+VF   +CF,
  (0x29<<8)      +YF   +XF      +CF,
  (0x2A<<8)      +YF   +XF      +CF,
  (0x2B<<8)      +YF   +XF+VF   +CF,
  (0x2C<<8)      +YF   +XF      +CF,
  (0x2D<<8)      +YF   +XF+VF   +CF,
  (0x2E<<8)      +YF   +XF+VF   +CF,
  (0x2F<<8)      +YF   +XF      +CF,
  (0x30<<8)      +YF+HF   +VF   +CF,
  (0x31<<8)      +YF+HF         +CF,
  (0x32<<8)      +YF+HF         +CF,
  (0x33<<8)      +YF+HF   +VF   +CF,
  (0x34<<8)      +YF+HF         +CF,
  (0x35<<8)      +YF+HF   +VF   +CF,
  (0x36<<8)      +YF      +VF   +CF,
  (0x37<<8)      +YF            +CF,
  (0x38<<8)      +YF   +XF      +CF,
  (0x39<<8)      +YF   +XF+VF   +CF,
  (0x3A<<8)      +YF   +XF+VF   +CF,
  (0x3B<<8)      +YF   +XF      +CF,
  (0x3C<<8)      +YF   +XF+VF   +CF,
  (0x3D<<8)      +YF   +XF      +CF,
  (0x3E<<8)      +YF   +XF      +CF,
  (0x3F<<8)      +YF   +XF+VF   +CF,
  (0x40<<8)         +HF         +CF,
  (0x41<<8)         +HF   +VF   +CF,
  (0x42<<8)         +HF   +VF   +CF,
  (0x43<<8)         +HF         +CF,
  (0x44<<8)         +HF   +VF   +CF,
  (0x45<<8)         +HF         +CF,
  (0x46<<8)                     +CF,
  (0x47<<8)               +VF   +CF,
  (0x48<<8)            +XF+VF   +CF,
  (0x49<<8)            +XF      +CF,
  (0x4A<<8)            +XF      +CF,
  (0x4B<<8)            +XF+VF   +CF,
  (0x4C<<8)            +XF      +CF,
  (0x4D<<8)            +XF+VF   +CF,
  (0x4E<<8)            +XF+VF   +CF,
  (0x4F<<8)            +XF      +CF,
  (0x50<<8)         +HF   +VF   +CF,
  (0x51<<8)         +HF         +CF,
  (0x52<<8)         +HF         +CF,
  (0x53<<8)         +HF   +VF   +CF,
  (0x54<<8)         +HF         +CF,
  (0x55<<8)         +HF   +VF   +CF,
  (0x56<<8)               +VF   +CF,
  (0x57<<8)                     +CF,
  (0x58<<8)            +XF      +CF,
  (0x59<<8)            +XF+VF   +CF,
  (0x5A<<8)            +XF+VF   +CF,
  (0x5B<<8)            +XF      +CF,
  (0x5C<<8)            +XF+VF   +CF,
  (0x5D<<8)            +XF      +CF,
  (0x5E<<8)            +XF      +CF,
  (0x5F<<8)            +XF+VF   +CF,
  (0x60<<8)      +YF+HF   +VF   +CF,
  (0x61<<8)      +YF+HF         +CF,
  (0x62<<8)      +YF+HF         +CF,
  (0x63<<8)      +YF+HF   +VF   +CF,
  (0x64<<8)      +YF+HF         +CF,
  (0x65<<8)      +YF+HF   +VF   +CF,
  (0x00<<8)   +ZF         +VF+NF   ,
  (0x01<<8)                  +NF   ,
  (0x02<<8)                  +NF   ,
  (0x03<<8)               +VF+NF   ,
  (0x04<<8)                  +NF   ,
  (0x05<<8)               +VF+NF   ,
  (0x06<<8)               +VF+NF   ,
  (0x07<<8)                  +NF   ,
  (0x08<<8)            +XF   +NF   ,
  (0x09<<8)            +XF+VF+NF   ,
  (0x04<<8)                  +NF   ,
  (0x05<<8)               +VF+NF   ,
  (0x06<<8)               +VF+NF   ,
  (0x07<<8)                  +NF   ,
  (0x08<<8)            +XF   +NF   ,
  (0x09<<8)            +XF+VF+NF   ,
  (0x10<<8)                  +NF   ,
  (0x11<<8)               +VF+NF   ,
  (0x12<<8)               +VF+NF   ,
  (0x13<<8)                  +NF   ,
  (0x14<<8)               +VF+NF   ,
  (0x15<<8)                  +NF   ,
  (0x16<<8)                  +NF   ,
  (0x17<<8)               +VF+NF   ,
  (0x18<<8)            +XF+VF+NF   ,
  (0x19<<8)            +XF   +NF   ,
  (0x14<<8)               +VF+NF   ,
  (0x15<<8)                  +NF   ,
  (0x16<<8)                  +NF   ,
  (0x17<<8)               +VF+NF   ,
  (0x18<<8)            +XF+VF+NF   ,
  (0x19<<8)            +XF   +NF   ,
  (0x20<<8)      +YF         +NF   ,
  (0x21<<8)      +YF      +VF+NF   ,
  (0x22<<8)      +YF      +VF+NF   ,
  (0x23<<8)      +YF         +NF   ,
  (0x24<<8)      +YF      +VF+NF   ,
  (0x25<<8)      +YF         +NF   ,
  (0x26<<8)      +YF         +NF   ,
  (0x27<<8)      +YF      +VF+NF   ,
  (0x28<<8)      +YF   +XF+VF+NF   ,
  (0x29<<8)      +YF   +XF   +NF   ,
  (0x24<<8)      +YF      +VF+NF   ,
  (0x25<<8)      +YF         +NF   ,
  (0x26<<8)      +YF         +NF   ,
  (0x27<<8)      +YF      +VF+NF   ,
  (0x28<<8)      +YF   +XF+VF+NF   ,
  (0x29<<8)      +YF   +XF   +NF   ,
  (0x30<<8)      +YF      +VF+NF   ,
  (0x31<<8)      +YF         +NF   ,
  (0x32<<8)      +YF         +NF   ,
  (0x33<<8)      +YF      +VF+NF   ,
  (0x34<<8)      +YF         +NF   ,
  (0x35<<8)      +YF      +VF+NF   ,
  (0x36<<8)      +YF      +VF+NF   ,
  (0x37<<8)      +YF         +NF   ,
  (0x38<<8)      +YF   +XF   +NF   ,
  (0x39<<8)      +YF   +XF+VF+NF   ,
  (0x34<<8)      +YF         +NF   ,
  (0x35<<8)      +YF      +VF+NF   ,
  (0x36<<8)      +YF      +VF+NF   ,
  (0x37<<8)      +YF         +NF   ,
  (0x38<<8)      +YF   +XF   +NF   ,
  (0x39<<8)      +YF   +XF+VF+NF   ,
  (0x40<<8)                  +NF   ,
  (0x41<<8)               +VF+NF   ,
  (0x42<<8)               +VF+NF   ,
  (0x43<<8)                  +NF   ,
  (0x44<<8)               +VF+NF   ,
  (0x45<<8)                  +NF   ,
  (0x46<<8)                  +NF   ,
  (0x47<<8)               +VF+NF   ,
  (0x48<<8)            +XF+VF+NF   ,
  (0x49<<8)            +XF   +NF   ,
  (0x44<<8)               +VF+NF   ,
  (0x45<<8)                  +NF   ,
  (0x46<<8)                  +NF   ,
  (0x47<<8)               +VF+NF   ,
  (0x48<<8)            +XF+VF+NF   ,
  (0x49<<8)            +XF   +NF   ,
  (0x50<<8)               +VF+NF   ,
  (0x51<<8)                  +NF   ,
  (0x52<<8)                  +NF   ,
  (0x53<<8)               +VF+NF   ,
  (0x54<<8)                  +NF   ,
  (0x55<<8)               +VF+NF   ,
  (0x56<<8)               +VF+NF   ,
  (0x57<<8)                  +NF   ,
  (0x58<<8)            +XF   +NF   ,
  (0x59<<8)            +XF+VF+NF   ,
  (0x54<<8)                  +NF   ,
  (0x55<<8)               +VF+NF   ,
  (0x56<<8)               +VF+NF   ,
  (0x57<<8)                  +NF   ,
  (0x58<<8)            +XF   +NF   ,
  (0x59<<8)            +XF+VF+NF   ,
  (0x60<<8)      +YF      +VF+NF   ,
  (0x61<<8)      +YF         +NF   ,
  (0x62<<8)      +YF         +NF   ,
  (0x63<<8)      +YF      +VF+NF   ,
  (0x64<<8)      +YF         +NF   ,
  (0x65<<8)      +YF      +VF+NF   ,
  (0x66<<8)      +YF      +VF+NF   ,
  (0x67<<8)      +YF         +NF   ,
  (0x68<<8)      +YF   +XF   +NF   ,
  (0x69<<8)      +YF   +XF+VF+NF   ,
  (0x64<<8)      +YF         +NF   ,
  (0x65<<8)      +YF      +VF+NF   ,
  (0x66<<8)      +YF      +VF+NF   ,
  (0x67<<8)      +YF         +NF   ,
  (0x68<<8)      +YF   +XF   +NF   ,
  (0x69<<8)      +YF   +XF+VF+NF   ,
  (0x70<<8)      +YF         +NF   ,
  (0x71<<8)      +YF      +VF+NF   ,
  (0x72<<8)      +YF      +VF+NF   ,
  (0x73<<8)      +YF         +NF   ,
  (0x74<<8)      +YF      +VF+NF   ,
  (0x75<<8)      +YF         +NF   ,
  (0x76<<8)      +YF         +NF   ,
  (0x77<<8)      +YF      +VF+NF   ,
  (0x78<<8)      +YF   +XF+VF+NF   ,
  (0x79<<8)      +YF   +XF   +NF   ,
  (0x74<<8)      +YF      +VF+NF   ,
  (0x75<<8)      +YF         +NF   ,
  (0x76<<8)      +YF         +NF   ,
  (0x77<<8)      +YF      +VF+NF   ,
  (0x78<<8)      +YF   +XF+VF+NF   ,
  (0x79<<8)      +YF   +XF   +NF   ,
  (0x80<<8)+SF               +NF   ,
  (0x81<<8)+SF            +VF+NF   ,
  (0x82<<8)+SF            +VF+NF   ,
  (0x83<<8)+SF               +NF   ,
  (0x84<<8)+SF            +VF+NF   ,
  (0x85<<8)+SF               +NF   ,
  (0x86<<8)+SF               +NF   ,
  (0x87<<8)+SF            +VF+NF   ,
  (0x88<<8)+SF         +XF+VF+NF   ,
  (0x89<<8)+SF         +XF   +NF   ,
  (0x84<<8)+SF            +VF+NF   ,
  (0x85<<8)+SF               +NF   ,
  (0x86<<8)+SF               +NF   ,
  (0x87<<8)+SF            +VF+NF   ,
  (0x88<<8)+SF         +XF+VF+NF   ,
  (0x89<<8)+SF         +XF   +NF   ,
  (0x90<<8)+SF            +VF+NF   ,
  (0x91<<8)+SF               +NF   ,
  (0x92<<8)+SF               +NF   ,
  (0x93<<8)+SF            +VF+NF   ,
  (0x94<<8)+SF               +NF   ,
  (0x95<<8)+SF            +VF+NF   ,
  (0x96<<8)+SF            +VF+NF   ,
  (0x97<<8)+SF               +NF   ,
  (0x98<<8)+SF         +XF   +NF   ,
  (0x99<<8)+SF         +XF+VF+NF   ,
  (0x34<<8)      +YF         +NF+CF,
  (0x35<<8)      +YF      +VF+NF+CF,
  (0x36<<8)      +YF      +VF+NF+CF,
  (0x37<<8)      +YF         +NF+CF,
  (0x38<<8)      +YF   +XF   +NF+CF,
  (0x39<<8)      +YF   +XF+VF+NF+CF,
  (0x40<<8)                  +NF+CF,
  (0x41<<8)               +VF+NF+CF,
  (0x42<<8)               +VF+NF+CF,
  (0x43<<8)                  +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x50<<8)               +VF+NF+CF,
  (0x51<<8)                  +NF+CF,
  (0x52<<8)                  +NF+CF,
  (0x53<<8)               +VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x60<<8)      +YF      +VF+NF+CF,
  (0x61<<8)      +YF         +NF+CF,
  (0x62<<8)      +YF         +NF+CF,
  (0x63<<8)      +YF      +VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x70<<8)      +YF         +NF+CF,
  (0x71<<8)      +YF      +VF+NF+CF,
  (0x72<<8)      +YF      +VF+NF+CF,
  (0x73<<8)      +YF         +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x80<<8)+SF               +NF+CF,
  (0x81<<8)+SF            +VF+NF+CF,
  (0x82<<8)+SF            +VF+NF+CF,
  (0x83<<8)+SF               +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x90<<8)+SF            +VF+NF+CF,
  (0x91<<8)+SF               +NF+CF,
  (0x92<<8)+SF               +NF+CF,
  (0x93<<8)+SF            +VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF,
  (0xA0<<8)+SF   +YF      +VF+NF+CF,
  (0xA1<<8)+SF   +YF         +NF+CF,
  (0xA2<<8)+SF   +YF         +NF+CF,
  (0xA3<<8)+SF   +YF      +VF+NF+CF,
  (0xA4<<8)+SF   +YF         +NF+CF,
  (0xA5<<8)+SF   +YF      +VF+NF+CF,
  (0xA6<<8)+SF   +YF      +VF+NF+CF,
  (0xA7<<8)+SF   +YF         +NF+CF,
  (0xA8<<8)+SF   +YF   +XF   +NF+CF,
  (0xA9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xA4<<8)+SF   +YF         +NF+CF,
  (0xA5<<8)+SF   +YF      +VF+NF+CF,
  (0xA6<<8)+SF   +YF      +VF+NF+CF,
  (0xA7<<8)+SF   +YF         +NF+CF,
  (0xA8<<8)+SF   +YF   +XF   +NF+CF,
  (0xA9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xB0<<8)+SF   +YF         +NF+CF,
  (0xB1<<8)+SF   +YF      +VF+NF+CF,
  (0xB2<<8)+SF   +YF      +VF+NF+CF,
  (0xB3<<8)+SF   +YF         +NF+CF,
  (0xB4<<8)+SF   +YF      +VF+NF+CF,
  (0xB5<<8)+SF   +YF         +NF+CF,
  (0xB6<<8)+SF   +YF         +NF+CF,
  (0xB7<<8)+SF   +YF      +VF+NF+CF,
  (0xB8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xB9<<8)+SF   +YF   +XF   +NF+CF,
  (0xB4<<8)+SF   +YF      +VF+NF+CF,
  (0xB5<<8)+SF   +YF         +NF+CF,
  (0xB6<<8)+SF   +YF         +NF+CF,
  (0xB7<<8)+SF   +YF      +VF+NF+CF,
  (0xB8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xB9<<8)+SF   +YF   +XF   +NF+CF,
  (0xC0<<8)+SF            +VF+NF+CF,
  (0xC1<<8)+SF               +NF+CF,
  (0xC2<<8)+SF               +NF+CF,
  (0xC3<<8)+SF            +VF+NF+CF,
  (0xC4<<8)+SF               +NF+CF,
  (0xC5<<8)+SF            +VF+NF+CF,
  (0xC6<<8)+SF            +VF+NF+CF,
  (0xC7<<8)+SF               +NF+CF,
  (0xC8<<8)+SF         +XF   +NF+CF,
  (0xC9<<8)+SF         +XF+VF+NF+CF,
  (0xC4<<8)+SF               +NF+CF,
  (0xC5<<8)+SF            +VF+NF+CF,
  (0xC6<<8)+SF            +VF+NF+CF,
  (0xC7<<8)+SF               +NF+CF,
  (0xC8<<8)+SF         +XF   +NF+CF,
  (0xC9<<8)+SF         +XF+VF+NF+CF,
  (0xD0<<8)+SF               +NF+CF,
  (0xD1<<8)+SF            +VF+NF+CF,
  (0xD2<<8)+SF            +VF+NF+CF,
  (0xD3<<8)+SF               +NF+CF,
  (0xD4<<8)+SF            +VF+NF+CF,
  (0xD5<<8)+SF               +NF+CF,
  (0xD6<<8)+SF               +NF+CF,
  (0xD7<<8)+SF            +VF+NF+CF,
  (0xD8<<8)+SF         +XF+VF+NF+CF,
  (0xD9<<8)+SF         +XF   +NF+CF,
  (0xD4<<8)+SF            +VF+NF+CF,
  (0xD5<<8)+SF               +NF+CF,
  (0xD6<<8)+SF               +NF+CF,
  (0xD7<<8)+SF            +VF+NF+CF,
  (0xD8<<8)+SF         +XF+VF+NF+CF,
  (0xD9<<8)+SF         +XF   +NF+CF,
  (0xE0<<8)+SF   +YF         +NF+CF,
  (0xE1<<8)+SF   +YF      +VF+NF+CF,
  (0xE2<<8)+SF   +YF      +VF+NF+CF,
  (0xE3<<8)+SF   +YF         +NF+CF,
  (0xE4<<8)+SF   +YF      +VF+NF+CF,
  (0xE5<<8)+SF   +YF         +NF+CF,
  (0xE6<<8)+SF   +YF         +NF+CF,
  (0xE7<<8)+SF   +YF      +VF+NF+CF,
  (0xE8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xE9<<8)+SF   +YF   +XF   +NF+CF,
  (0xE4<<8)+SF   +YF      +VF+NF+CF,
  (0xE5<<8)+SF   +YF         +NF+CF,
  (0xE6<<8)+SF   +YF         +NF+CF,
  (0xE7<<8)+SF   +YF      +VF+NF+CF,
  (0xE8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xE9<<8)+SF   +YF   +XF   +NF+CF,
  (0xF0<<8)+SF   +YF      +VF+NF+CF,
  (0xF1<<8)+SF   +YF         +NF+CF,
  (0xF2<<8)+SF   +YF         +NF+CF,
  (0xF3<<8)+SF   +YF      +VF+NF+CF,
  (0xF4<<8)+SF   +YF         +NF+CF,
  (0xF5<<8)+SF   +YF      +VF+NF+CF,
  (0xF6<<8)+SF   +YF      +VF+NF+CF,
  (0xF7<<8)+SF   +YF         +NF+CF,
  (0xF8<<8)+SF   +YF   +XF   +NF+CF,
  (0xF9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xF4<<8)+SF   +YF         +NF+CF,
  (0xF5<<8)+SF   +YF      +VF+NF+CF,
  (0xF6<<8)+SF   +YF      +VF+NF+CF,
  (0xF7<<8)+SF   +YF         +NF+CF,
  (0xF8<<8)+SF   +YF   +XF   +NF+CF,
  (0xF9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0x00<<8)   +ZF         +VF+NF+CF,
  (0x01<<8)                  +NF+CF,
  (0x02<<8)                  +NF+CF,
  (0x03<<8)               +VF+NF+CF,
  (0x04<<8)                  +NF+CF,
  (0x05<<8)               +VF+NF+CF,
  (0x06<<8)               +VF+NF+CF,
  (0x07<<8)                  +NF+CF,
  (0x08<<8)            +XF   +NF+CF,
  (0x09<<8)            +XF+VF+NF+CF,
  (0x04<<8)                  +NF+CF,
  (0x05<<8)               +VF+NF+CF,
  (0x06<<8)               +VF+NF+CF,
  (0x07<<8)                  +NF+CF,
  (0x08<<8)            +XF   +NF+CF,
  (0x09<<8)            +XF+VF+NF+CF,
  (0x10<<8)                  +NF+CF,
  (0x11<<8)               +VF+NF+CF,
  (0x12<<8)               +VF+NF+CF,
  (0x13<<8)                  +NF+CF,
  (0x14<<8)               +VF+NF+CF,
  (0x15<<8)                  +NF+CF,
  (0x16<<8)                  +NF+CF,
  (0x17<<8)               +VF+NF+CF,
  (0x18<<8)            +XF+VF+NF+CF,
  (0x19<<8)            +XF   +NF+CF,
  (0x14<<8)               +VF+NF+CF,
  (0x15<<8)                  +NF+CF,
  (0x16<<8)                  +NF+CF,
  (0x17<<8)               +VF+NF+CF,
  (0x18<<8)            +XF+VF+NF+CF,
  (0x19<<8)            +XF   +NF+CF,
  (0x20<<8)      +YF         +NF+CF,
  (0x21<<8)      +YF      +VF+NF+CF,
  (0x22<<8)      +YF      +VF+NF+CF,
  (0x23<<8)      +YF         +NF+CF,
  (0x24<<8)      +YF      +VF+NF+CF,
  (0x25<<8)      +YF         +NF+CF,
  (0x26<<8)      +YF         +NF+CF,
  (0x27<<8)      +YF      +VF+NF+CF,
  (0x28<<8)      +YF   +XF+VF+NF+CF,
  (0x29<<8)      +YF   +XF   +NF+CF,
  (0x24<<8)      +YF      +VF+NF+CF,
  (0x25<<8)      +YF         +NF+CF,
  (0x26<<8)      +YF         +NF+CF,
  (0x27<<8)      +YF      +VF+NF+CF,
  (0x28<<8)      +YF   +XF+VF+NF+CF,
  (0x29<<8)      +YF   +XF   +NF+CF,
  (0x30<<8)      +YF      +VF+NF+CF,
  (0x31<<8)      +YF         +NF+CF,
  (0x32<<8)      +YF         +NF+CF,
  (0x33<<8)      +YF      +VF+NF+CF,
  (0x34<<8)      +YF         +NF+CF,
  (0x35<<8)      +YF      +VF+NF+CF,
  (0x36<<8)      +YF      +VF+NF+CF,
  (0x37<<8)      +YF         +NF+CF,
  (0x38<<8)      +YF   +XF   +NF+CF,
  (0x39<<8)      +YF   +XF+VF+NF+CF,
  (0x34<<8)      +YF         +NF+CF,
  (0x35<<8)      +YF      +VF+NF+CF,
  (0x36<<8)      +YF      +VF+NF+CF,
  (0x37<<8)      +YF         +NF+CF,
  (0x38<<8)      +YF   +XF   +NF+CF,
  (0x39<<8)      +YF   +XF+VF+NF+CF,
  (0x40<<8)                  +NF+CF,
  (0x41<<8)               +VF+NF+CF,
  (0x42<<8)               +VF+NF+CF,
  (0x43<<8)                  +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x50<<8)               +VF+NF+CF,
  (0x51<<8)                  +NF+CF,
  (0x52<<8)                  +NF+CF,
  (0x53<<8)               +VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x60<<8)      +YF      +VF+NF+CF,
  (0x61<<8)      +YF         +NF+CF,
  (0x62<<8)      +YF         +NF+CF,
  (0x63<<8)      +YF      +VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x70<<8)      +YF         +NF+CF,
  (0x71<<8)      +YF      +VF+NF+CF,
  (0x72<<8)      +YF      +VF+NF+CF,
  (0x73<<8)      +YF         +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x80<<8)+SF               +NF+CF,
  (0x81<<8)+SF            +VF+NF+CF,
  (0x82<<8)+SF            +VF+NF+CF,
  (0x83<<8)+SF               +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x90<<8)+SF            +VF+NF+CF,
  (0x91<<8)+SF               +NF+CF,
  (0x92<<8)+SF               +NF+CF,
  (0x93<<8)+SF            +VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF,
  (0xFA<<8)+SF   +YF+HF+XF+VF+NF   ,
  (0xFB<<8)+SF   +YF+HF+XF   +NF   ,
  (0xFC<<8)+SF   +YF+HF+XF+VF+NF   ,
  (0xFD<<8)+SF   +YF+HF+XF   +NF   ,
  (0xFE<<8)+SF   +YF+HF+XF   +NF   ,
  (0xFF<<8)+SF   +YF+HF+XF+VF+NF   ,
  (0x00<<8)   +ZF         +VF+NF   ,
  (0x01<<8)                  +NF   ,
  (0x02<<8)                  +NF   ,
  (0x03<<8)               +VF+NF   ,
  (0x04<<8)                  +NF   ,
  (0x05<<8)               +VF+NF   ,
  (0x06<<8)               +VF+NF   ,
  (0x07<<8)                  +NF   ,
  (0x08<<8)            +XF   +NF   ,
  (0x09<<8)            +XF+VF+NF   ,
  (0x0A<<8)         +HF+XF+VF+NF   ,
  (0x0B<<8)         +HF+XF   +NF   ,
  (0x0C<<8)         +HF+XF+VF+NF   ,
  (0x0D<<8)         +HF+XF   +NF   ,
  (0x0E<<8)         +HF+XF   +NF   ,
  (0x0F<<8)         +HF+XF+VF+NF   ,
  (0x10<<8)                  +NF   ,
  (0x11<<8)               +VF+NF   ,
  (0x12<<8)               +VF+NF   ,
  (0x13<<8)                  +NF   ,
  (0x14<<8)               +VF+NF   ,
  (0x15<<8)                  +NF   ,
  (0x16<<8)                  +NF   ,
  (0x17<<8)               +VF+NF   ,
  (0x18<<8)            +XF+VF+NF   ,
  (0x19<<8)            +XF   +NF   ,
  (0x1A<<8)         +HF+XF   +NF   ,
  (0x1B<<8)         +HF+XF+VF+NF   ,
  (0x1C<<8)         +HF+XF   +NF   ,
  (0x1D<<8)         +HF+XF+VF+NF   ,
  (0x1E<<8)         +HF+XF+VF+NF   ,
  (0x1F<<8)         +HF+XF   +NF   ,
  (0x20<<8)      +YF         +NF   ,
  (0x21<<8)      +YF      +VF+NF   ,
  (0x22<<8)      +YF      +VF+NF   ,
  (0x23<<8)      +YF         +NF   ,
  (0x24<<8)      +YF      +VF+NF   ,
  (0x25<<8)      +YF         +NF   ,
  (0x26<<8)      +YF         +NF   ,
  (0x27<<8)      +YF      +VF+NF   ,
  (0x28<<8)      +YF   +XF+VF+NF   ,
  (0x29<<8)      +YF   +XF   +NF   ,
  (0x2A<<8)      +YF+HF+XF   +NF   ,
  (0x2B<<8)      +YF+HF+XF+VF+NF   ,
  (0x2C<<8)      +YF+HF+XF   +NF   ,
  (0x2D<<8)      +YF+HF+XF+VF+NF   ,
  (0x2E<<8)      +YF+HF+XF+VF+NF   ,
  (0x2F<<8)      +YF+HF+XF   +NF   ,
  (0x30<<8)      +YF      +VF+NF   ,
  (0x31<<8)      +YF         +NF   ,
  (0x32<<8)      +YF         +NF   ,
  (0x33<<8)      +YF      +VF+NF   ,
  (0x34<<8)      +YF         +NF   ,
  (0x35<<8)      +YF      +VF+NF   ,
  (0x36<<8)      +YF      +VF+NF   ,
  (0x37<<8)      +YF         +NF   ,
  (0x38<<8)      +YF   +XF   +NF   ,
  (0x39<<8)      +YF   +XF+VF+NF   ,
  (0x3A<<8)      +YF+HF+XF+VF+NF   ,
  (0x3B<<8)      +YF+HF+XF   +NF   ,
  (0x3C<<8)      +YF+HF+XF+VF+NF   ,
  (0x3D<<8)      +YF+HF+XF   +NF   ,
  (0x3E<<8)      +YF+HF+XF   +NF   ,
  (0x3F<<8)      +YF+HF+XF+VF+NF   ,
  (0x40<<8)                  +NF   ,
  (0x41<<8)               +VF+NF   ,
  (0x42<<8)               +VF+NF   ,
  (0x43<<8)                  +NF   ,
  (0x44<<8)               +VF+NF   ,
  (0x45<<8)                  +NF   ,
  (0x46<<8)                  +NF   ,
  (0x47<<8)               +VF+NF   ,
  (0x48<<8)            +XF+VF+NF   ,
  (0x49<<8)            +XF   +NF   ,
  (0x4A<<8)         +HF+XF   +NF   ,
  (0x4B<<8)         +HF+XF+VF+NF   ,
  (0x4C<<8)         +HF+XF   +NF   ,
  (0x4D<<8)         +HF+XF+VF+NF   ,
  (0x4E<<8)         +HF+XF+VF+NF   ,
  (0x4F<<8)         +HF+XF   +NF   ,
  (0x50<<8)               +VF+NF   ,
  (0x51<<8)                  +NF   ,
  (0x52<<8)                  +NF   ,
  (0x53<<8)               +VF+NF   ,
  (0x54<<8)                  +NF   ,
  (0x55<<8)               +VF+NF   ,
  (0x56<<8)               +VF+NF   ,
  (0x57<<8)                  +NF   ,
  (0x58<<8)            +XF   +NF   ,
  (0x59<<8)            +XF+VF+NF   ,
  (0x5A<<8)         +HF+XF+VF+NF   ,
  (0x5B<<8)         +HF+XF   +NF   ,
  (0x5C<<8)         +HF+XF+VF+NF   ,
  (0x5D<<8)         +HF+XF   +NF   ,
  (0x5E<<8)         +HF+XF   +NF   ,
  (0x5F<<8)         +HF+XF+VF+NF   ,
  (0x60<<8)      +YF      +VF+NF   ,
  (0x61<<8)      +YF         +NF   ,
  (0x62<<8)      +YF         +NF   ,
  (0x63<<8)      +YF      +VF+NF   ,
  (0x64<<8)      +YF         +NF   ,
  (0x65<<8)      +YF      +VF+NF   ,
  (0x66<<8)      +YF      +VF+NF   ,
  (0x67<<8)      +YF         +NF   ,
  (0x68<<8)      +YF   +XF   +NF   ,
  (0x69<<8)      +YF   +XF+VF+NF   ,
  (0x6A<<8)      +YF+HF+XF+VF+NF   ,
  (0x6B<<8)      +YF+HF+XF   +NF   ,
  (0x6C<<8)      +YF+HF+XF+VF+NF   ,
  (0x6D<<8)      +YF+HF+XF   +NF   ,
  (0x6E<<8)      +YF+HF+XF   +NF   ,
  (0x6F<<8)      +YF+HF+XF+VF+NF   ,
  (0x70<<8)      +YF         +NF   ,
  (0x71<<8)      +YF      +VF+NF   ,
  (0x72<<8)      +YF      +VF+NF   ,
  (0x73<<8)      +YF         +NF   ,
  (0x74<<8)      +YF      +VF+NF   ,
  (0x75<<8)      +YF         +NF   ,
  (0x76<<8)      +YF         +NF   ,
  (0x77<<8)      +YF      +VF+NF   ,
  (0x78<<8)      +YF   +XF+VF+NF   ,
  (0x79<<8)      +YF   +XF   +NF   ,
  (0x7A<<8)      +YF+HF+XF   +NF   ,
  (0x7B<<8)      +YF+HF+XF+VF+NF   ,
  (0x7C<<8)      +YF+HF+XF   +NF   ,
  (0x7D<<8)      +YF+HF+XF+VF+NF   ,
  (0x7E<<8)      +YF+HF+XF+VF+NF   ,
  (0x7F<<8)      +YF+HF+XF   +NF   ,
  (0x80<<8)+SF               +NF   ,
  (0x81<<8)+SF            +VF+NF   ,
  (0x82<<8)+SF            +VF+NF   ,
  (0x83<<8)+SF               +NF   ,
  (0x84<<8)+SF            +VF+NF   ,
  (0x85<<8)+SF               +NF   ,
  (0x86<<8)+SF               +NF   ,
  (0x87<<8)+SF            +VF+NF   ,
  (0x88<<8)+SF         +XF+VF+NF   ,
  (0x89<<8)+SF         +XF   +NF   ,
  (0x8A<<8)+SF      +HF+XF   +NF   ,
  (0x8B<<8)+SF      +HF+XF+VF+NF   ,
  (0x8C<<8)+SF      +HF+XF   +NF   ,
  (0x8D<<8)+SF      +HF+XF+VF+NF   ,
  (0x8E<<8)+SF      +HF+XF+VF+NF   ,
  (0x8F<<8)+SF      +HF+XF   +NF   ,
  (0x90<<8)+SF            +VF+NF   ,
  (0x91<<8)+SF               +NF   ,
  (0x92<<8)+SF               +NF   ,
  (0x93<<8)+SF            +VF+NF   ,
  (0x34<<8)      +YF         +NF+CF,
  (0x35<<8)      +YF      +VF+NF+CF,
  (0x36<<8)      +YF      +VF+NF+CF,
  (0x37<<8)      +YF         +NF+CF,
  (0x38<<8)      +YF   +XF   +NF+CF,
  (0x39<<8)      +YF   +XF+VF+NF+CF,
  (0x3A<<8)      +YF+HF+XF+VF+NF+CF,
  (0x3B<<8)      +YF+HF+XF   +NF+CF,
  (0x3C<<8)      +YF+HF+XF+VF+NF+CF,
  (0x3D<<8)      +YF+HF+XF   +NF+CF,
  (0x3E<<8)      +YF+HF+XF   +NF+CF,
  (0x3F<<8)      +YF+HF+XF+VF+NF+CF,
  (0x40<<8)                  +NF+CF,
  (0x41<<8)               +VF+NF+CF,
  (0x42<<8)               +VF+NF+CF,
  (0x43<<8)                  +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x4A<<8)         +HF+XF   +NF+CF,
  (0x4B<<8)         +HF+XF+VF+NF+CF,
  (0x4C<<8)         +HF+XF   +NF+CF,
  (0x4D<<8)         +HF+XF+VF+NF+CF,
  (0x4E<<8)         +HF+XF+VF+NF+CF,
  (0x4F<<8)         +HF+XF   +NF+CF,
  (0x50<<8)               +VF+NF+CF,
  (0x51<<8)                  +NF+CF,
  (0x52<<8)                  +NF+CF,
  (0x53<<8)               +VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x5A<<8)         +HF+XF+VF+NF+CF,
  (0x5B<<8)         +HF+XF   +NF+CF,
  (0x5C<<8)         +HF+XF+VF+NF+CF,
  (0x5D<<8)         +HF+XF   +NF+CF,
  (0x5E<<8)         +HF+XF   +NF+CF,
  (0x5F<<8)         +HF+XF+VF+NF+CF,
  (0x60<<8)      +YF      +VF+NF+CF,
  (0x61<<8)      +YF         +NF+CF,
  (0x62<<8)      +YF         +NF+CF,
  (0x63<<8)      +YF      +VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x6A<<8)      +YF+HF+XF+VF+NF+CF,
  (0x6B<<8)      +YF+HF+XF   +NF+CF,
  (0x6C<<8)      +YF+HF+XF+VF+NF+CF,
  (0x6D<<8)      +YF+HF+XF   +NF+CF,
  (0x6E<<8)      +YF+HF+XF   +NF+CF,
  (0x6F<<8)      +YF+HF+XF+VF+NF+CF,
  (0x70<<8)      +YF         +NF+CF,
  (0x71<<8)      +YF      +VF+NF+CF,
  (0x72<<8)      +YF      +VF+NF+CF,
  (0x73<<8)      +YF         +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x7A<<8)      +YF+HF+XF   +NF+CF,
  (0x7B<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7C<<8)      +YF+HF+XF   +NF+CF,
  (0x7D<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7E<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7F<<8)      +YF+HF+XF   +NF+CF,
  (0x80<<8)+SF               +NF+CF,
  (0x81<<8)+SF            +VF+NF+CF,
  (0x82<<8)+SF            +VF+NF+CF,
  (0x83<<8)+SF               +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x8A<<8)+SF      +HF+XF   +NF+CF,
  (0x8B<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8C<<8)+SF      +HF+XF   +NF+CF,
  (0x8D<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8E<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8F<<8)+SF      +HF+XF   +NF+CF,
  (0x90<<8)+SF            +VF+NF+CF,
  (0x91<<8)+SF               +NF+CF,
  (0x92<<8)+SF               +NF+CF,
  (0x93<<8)+SF            +VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF,
  (0x9A<<8)+SF      +HF+XF+VF+NF+CF,
  (0x9B<<8)+SF      +HF+XF   +NF+CF,
  (0x9C<<8)+SF      +HF+XF+VF+NF+CF,
  (0x9D<<8)+SF      +HF+XF   +NF+CF,
  (0x9E<<8)+SF      +HF+XF   +NF+CF,
  (0x9F<<8)+SF      +HF+XF+VF+NF+CF,
  (0xA0<<8)+SF   +YF      +VF+NF+CF,
  (0xA1<<8)+SF   +YF         +NF+CF,
  (0xA2<<8)+SF   +YF         +NF+CF,
  (0xA3<<8)+SF   +YF      +VF+NF+CF,
  (0xA4<<8)+SF   +YF         +NF+CF,
  (0xA5<<8)+SF   +YF      +VF+NF+CF,
  (0xA6<<8)+SF   +YF      +VF+NF+CF,
  (0xA7<<8)+SF   +YF         +NF+CF,
  (0xA8<<8)+SF   +YF   +XF   +NF+CF,
  (0xA9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xAA<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xAB<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xAC<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xAD<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xAE<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xAF<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xB0<<8)+SF   +YF         +NF+CF,
  (0xB1<<8)+SF   +YF      +VF+NF+CF,
  (0xB2<<8)+SF   +YF      +VF+NF+CF,
  (0xB3<<8)+SF   +YF         +NF+CF,
  (0xB4<<8)+SF   +YF      +VF+NF+CF,
  (0xB5<<8)+SF   +YF         +NF+CF,
  (0xB6<<8)+SF   +YF         +NF+CF,
  (0xB7<<8)+SF   +YF      +VF+NF+CF,
  (0xB8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xB9<<8)+SF   +YF   +XF   +NF+CF,
  (0xBA<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xBB<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xBC<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xBD<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xBE<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xBF<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xC0<<8)+SF            +VF+NF+CF,
  (0xC1<<8)+SF               +NF+CF,
  (0xC2<<8)+SF               +NF+CF,
  (0xC3<<8)+SF            +VF+NF+CF,
  (0xC4<<8)+SF               +NF+CF,
  (0xC5<<8)+SF            +VF+NF+CF,
  (0xC6<<8)+SF            +VF+NF+CF,
  (0xC7<<8)+SF               +NF+CF,
  (0xC8<<8)+SF         +XF   +NF+CF,
  (0xC9<<8)+SF         +XF+VF+NF+CF,
  (0xCA<<8)+SF      +HF+XF+VF+NF+CF,
  (0xCB<<8)+SF      +HF+XF   +NF+CF,
  (0xCC<<8)+SF      +HF+XF+VF+NF+CF,
  (0xCD<<8)+SF      +HF+XF   +NF+CF,
  (0xCE<<8)+SF      +HF+XF   +NF+CF,
  (0xCF<<8)+SF      +HF+XF+VF+NF+CF,
  (0xD0<<8)+SF               +NF+CF,
  (0xD1<<8)+SF            +VF+NF+CF,
  (0xD2<<8)+SF            +VF+NF+CF,
  (0xD3<<8)+SF               +NF+CF,
  (0xD4<<8)+SF            +VF+NF+CF,
  (0xD5<<8)+SF               +NF+CF,
  (0xD6<<8)+SF               +NF+CF,
  (0xD7<<8)+SF            +VF+NF+CF,
  (0xD8<<8)+SF         +XF+VF+NF+CF,
  (0xD9<<8)+SF         +XF   +NF+CF,
  (0xDA<<8)+SF      +HF+XF   +NF+CF,
  (0xDB<<8)+SF      +HF+XF+VF+NF+CF,
  (0xDC<<8)+SF      +HF+XF   +NF+CF,
  (0xDD<<8)+SF      +HF+XF+VF+NF+CF,
  (0xDE<<8)+SF      +HF+XF+VF+NF+CF,
  (0xDF<<8)+SF      +HF+XF   +NF+CF,
  (0xE0<<8)+SF   +YF         +NF+CF,
  (0xE1<<8)+SF   +YF      +VF+NF+CF,
  (0xE2<<8)+SF   +YF      +VF+NF+CF,
  (0xE3<<8)+SF   +YF         +NF+CF,
  (0xE4<<8)+SF   +YF      +VF+NF+CF,
  (0xE5<<8)+SF   +YF         +NF+CF,
  (0xE6<<8)+SF   +YF         +NF+CF,
  (0xE7<<8)+SF   +YF      +VF+NF+CF,
  (0xE8<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xE9<<8)+SF   +YF   +XF   +NF+CF,
  (0xEA<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xEB<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xEC<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xED<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xEE<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xEF<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xF0<<8)+SF   +YF      +VF+NF+CF,
  (0xF1<<8)+SF   +YF         +NF+CF,
  (0xF2<<8)+SF   +YF         +NF+CF,
  (0xF3<<8)+SF   +YF      +VF+NF+CF,
  (0xF4<<8)+SF   +YF         +NF+CF,
  (0xF5<<8)+SF   +YF      +VF+NF+CF,
  (0xF6<<8)+SF   +YF      +VF+NF+CF,
  (0xF7<<8)+SF   +YF         +NF+CF,
  (0xF8<<8)+SF   +YF   +XF   +NF+CF,
  (0xF9<<8)+SF   +YF   +XF+VF+NF+CF,
  (0xFA<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xFB<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xFC<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0xFD<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xFE<<8)+SF   +YF+HF+XF   +NF+CF,
  (0xFF<<8)+SF   +YF+HF+XF+VF+NF+CF,
  (0x00<<8)   +ZF         +VF+NF+CF,
  (0x01<<8)                  +NF+CF,
  (0x02<<8)                  +NF+CF,
  (0x03<<8)               +VF+NF+CF,
  (0x04<<8)                  +NF+CF,
  (0x05<<8)               +VF+NF+CF,
  (0x06<<8)               +VF+NF+CF,
  (0x07<<8)                  +NF+CF,
  (0x08<<8)            +XF   +NF+CF,
  (0x09<<8)            +XF+VF+NF+CF,
  (0x0A<<8)         +HF+XF+VF+NF+CF,
  (0x0B<<8)         +HF+XF   +NF+CF,
  (0x0C<<8)         +HF+XF+VF+NF+CF,
  (0x0D<<8)         +HF+XF   +NF+CF,
  (0x0E<<8)         +HF+XF   +NF+CF,
  (0x0F<<8)         +HF+XF+VF+NF+CF,
  (0x10<<8)                  +NF+CF,
  (0x11<<8)               +VF+NF+CF,
  (0x12<<8)               +VF+NF+CF,
  (0x13<<8)                  +NF+CF,
  (0x14<<8)               +VF+NF+CF,
  (0x15<<8)                  +NF+CF,
  (0x16<<8)                  +NF+CF,
  (0x17<<8)               +VF+NF+CF,
  (0x18<<8)            +XF+VF+NF+CF,
  (0x19<<8)            +XF   +NF+CF,
  (0x1A<<8)         +HF+XF   +NF+CF,
  (0x1B<<8)         +HF+XF+VF+NF+CF,
  (0x1C<<8)         +HF+XF   +NF+CF,
  (0x1D<<8)         +HF+XF+VF+NF+CF,
  (0x1E<<8)         +HF+XF+VF+NF+CF,
  (0x1F<<8)         +HF+XF   +NF+CF,
  (0x20<<8)      +YF         +NF+CF,
  (0x21<<8)      +YF      +VF+NF+CF,
  (0x22<<8)      +YF      +VF+NF+CF,
  (0x23<<8)      +YF         +NF+CF,
  (0x24<<8)      +YF      +VF+NF+CF,
  (0x25<<8)      +YF         +NF+CF,
  (0x26<<8)      +YF         +NF+CF,
  (0x27<<8)      +YF      +VF+NF+CF,
  (0x28<<8)      +YF   +XF+VF+NF+CF,
  (0x29<<8)      +YF   +XF   +NF+CF,
  (0x2A<<8)      +YF+HF+XF   +NF+CF,
  (0x2B<<8)      +YF+HF+XF+VF+NF+CF,
  (0x2C<<8)      +YF+HF+XF   +NF+CF,
  (0x2D<<8)      +YF+HF+XF+VF+NF+CF,
  (0x2E<<8)      +YF+HF+XF+VF+NF+CF,
  (0x2F<<8)      +YF+HF+XF   +NF+CF,
  (0x30<<8)      +YF      +VF+NF+CF,
  (0x31<<8)      +YF         +NF+CF,
  (0x32<<8)      +YF         +NF+CF,
  (0x33<<8)      +YF      +VF+NF+CF,
  (0x34<<8)      +YF         +NF+CF,
  (0x35<<8)      +YF      +VF+NF+CF,
  (0x36<<8)      +YF      +VF+NF+CF,
  (0x37<<8)      +YF         +NF+CF,
  (0x38<<8)      +YF   +XF   +NF+CF,
  (0x39<<8)      +YF   +XF+VF+NF+CF,
  (0x3A<<8)      +YF+HF+XF+VF+NF+CF,
  (0x3B<<8)      +YF+HF+XF   +NF+CF,
  (0x3C<<8)      +YF+HF+XF+VF+NF+CF,
  (0x3D<<8)      +YF+HF+XF   +NF+CF,
  (0x3E<<8)      +YF+HF+XF   +NF+CF,
  (0x3F<<8)      +YF+HF+XF+VF+NF+CF,
  (0x40<<8)                  +NF+CF,
  (0x41<<8)               +VF+NF+CF,
  (0x42<<8)               +VF+NF+CF,
  (0x43<<8)                  +NF+CF,
  (0x44<<8)               +VF+NF+CF,
  (0x45<<8)                  +NF+CF,
  (0x46<<8)                  +NF+CF,
  (0x47<<8)               +VF+NF+CF,
  (0x48<<8)            +XF+VF+NF+CF,
  (0x49<<8)            +XF   +NF+CF,
  (0x4A<<8)         +HF+XF   +NF+CF,
  (0x4B<<8)         +HF+XF+VF+NF+CF,
  (0x4C<<8)         +HF+XF   +NF+CF,
  (0x4D<<8)         +HF+XF+VF+NF+CF,
  (0x4E<<8)         +HF+XF+VF+NF+CF,
  (0x4F<<8)         +HF+XF   +NF+CF,
  (0x50<<8)               +VF+NF+CF,
  (0x51<<8)                  +NF+CF,
  (0x52<<8)                  +NF+CF,
  (0x53<<8)               +VF+NF+CF,
  (0x54<<8)                  +NF+CF,
  (0x55<<8)               +VF+NF+CF,
  (0x56<<8)               +VF+NF+CF,
  (0x57<<8)                  +NF+CF,
  (0x58<<8)            +XF   +NF+CF,
  (0x59<<8)            +XF+VF+NF+CF,
  (0x5A<<8)         +HF+XF+VF+NF+CF,
  (0x5B<<8)         +HF+XF   +NF+CF,
  (0x5C<<8)         +HF+XF+VF+NF+CF,
  (0x5D<<8)         +HF+XF   +NF+CF,
  (0x5E<<8)         +HF+XF   +NF+CF,
  (0x5F<<8)         +HF+XF+VF+NF+CF,
  (0x60<<8)      +YF      +VF+NF+CF,
  (0x61<<8)      +YF         +NF+CF,
  (0x62<<8)      +YF         +NF+CF,
  (0x63<<8)      +YF      +VF+NF+CF,
  (0x64<<8)      +YF         +NF+CF,
  (0x65<<8)      +YF      +VF+NF+CF,
  (0x66<<8)      +YF      +VF+NF+CF,
  (0x67<<8)      +YF         +NF+CF,
  (0x68<<8)      +YF   +XF   +NF+CF,
  (0x69<<8)      +YF   +XF+VF+NF+CF,
  (0x6A<<8)      +YF+HF+XF+VF+NF+CF,
  (0x6B<<8)      +YF+HF+XF   +NF+CF,
  (0x6C<<8)      +YF+HF+XF+VF+NF+CF,
  (0x6D<<8)      +YF+HF+XF   +NF+CF,
  (0x6E<<8)      +YF+HF+XF   +NF+CF,
  (0x6F<<8)      +YF+HF+XF+VF+NF+CF,
  (0x70<<8)      +YF         +NF+CF,
  (0x71<<8)      +YF      +VF+NF+CF,
  (0x72<<8)      +YF      +VF+NF+CF,
  (0x73<<8)      +YF         +NF+CF,
  (0x74<<8)      +YF      +VF+NF+CF,
  (0x75<<8)      +YF         +NF+CF,
  (0x76<<8)      +YF         +NF+CF,
  (0x77<<8)      +YF      +VF+NF+CF,
  (0x78<<8)      +YF   +XF+VF+NF+CF,
  (0x79<<8)      +YF   +XF   +NF+CF,
  (0x7A<<8)      +YF+HF+XF   +NF+CF,
  (0x7B<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7C<<8)      +YF+HF+XF   +NF+CF,
  (0x7D<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7E<<8)      +YF+HF+XF+VF+NF+CF,
  (0x7F<<8)      +YF+HF+XF   +NF+CF,
  (0x80<<8)+SF               +NF+CF,
  (0x81<<8)+SF            +VF+NF+CF,
  (0x82<<8)+SF            +VF+NF+CF,
  (0x83<<8)+SF               +NF+CF,
  (0x84<<8)+SF            +VF+NF+CF,
  (0x85<<8)+SF               +NF+CF,
  (0x86<<8)+SF               +NF+CF,
  (0x87<<8)+SF            +VF+NF+CF,
  (0x88<<8)+SF         +XF+VF+NF+CF,
  (0x89<<8)+SF         +XF   +NF+CF,
  (0x8A<<8)+SF      +HF+XF   +NF+CF,
  (0x8B<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8C<<8)+SF      +HF+XF   +NF+CF,
  (0x8D<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8E<<8)+SF      +HF+XF+VF+NF+CF,
  (0x8F<<8)+SF      +HF+XF   +NF+CF,
  (0x90<<8)+SF            +VF+NF+CF,
  (0x91<<8)+SF               +NF+CF,
  (0x92<<8)+SF               +NF+CF,
  (0x93<<8)+SF            +VF+NF+CF,
  (0x94<<8)+SF               +NF+CF,
  (0x95<<8)+SF            +VF+NF+CF,
  (0x96<<8)+SF            +VF+NF+CF,
  (0x97<<8)+SF               +NF+CF,
  (0x98<<8)+SF         +XF   +NF+CF,
  (0x99<<8)+SF         +XF+VF+NF+CF
};

